package pl.primesoft.aztecreaderdemo.Scanning.CameraManager.Utils;

import android.hardware.Camera;
import android.os.Build;
import android.view.View;

import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import pl.primesoft.aztecreaderdemo.Scanning.CameraManager.CameraException;
import pl.primesoft.aztecreaderdemo.Scanning.CameraManager.CameraManager;

/**
 * Created by wygralak on 2016-03-15.
 */
public class AutoFocusUtils {

    public Timer focusTimer;
    public boolean refocusingActive = false;
    private CameraManager.AutoFocusMode currentAutoFocus;

    public View.OnClickListener setupContinuousAutoFocus(final Camera camera) {
        currentAutoFocus = CameraManager.AutoFocusMode.CONTINUOUS_MODE;
        List<String> supportedFocusModes = camera.getParameters().getSupportedFocusModes();
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.ICE_CREAM_SANDWICH || !supportedFocusModes.contains(Camera.Parameters.FOCUS_MODE_CONTINUOUS_PICTURE)) {
            startTimerFocusing(camera);
        } else {
            try {
                enableContinuousAutoFocus(camera);
                return new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        disableContinuousAutoFocus(camera);
                        enableContinuousAutoFocus(camera);
                    }
                };
            } catch (Exception e) {
                e.printStackTrace();
                currentAutoFocus = CameraManager.AutoFocusMode.TIMER_MODE;
                startTimerFocusing(camera);
                return null;
            }
        }
        return null;
    }


    public void startTimerFocusing(final Camera camera) {
        currentAutoFocus = CameraManager.AutoFocusMode.TIMER_MODE;
        if (refocusingActive) {
            return;
        }
        refocusingActive = true;

        focusTimer = new Timer();
        focusTimer.schedule(new TimerTask() {

            @Override
            public void run() {
                requestSingleAutoFocus(camera);
            }
        }, 500, CameraManager.REFOCUSING_DELAY);
    }

    public void requestSingleAutoFocus(Camera camera){
        if (camera != null) {
            try {
                camera.autoFocus(null);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    public void stopTimerFocusing() {
        if (!refocusingActive) {
            return;
        }

        if (focusTimer != null) {
            focusTimer.cancel();
            focusTimer.purge();
        }

        refocusingActive = false;
    }

    private void enableContinuousAutoFocus(Camera camera) {
        Camera.Parameters parameters = camera.getParameters();
        parameters.setFocusMode(Camera.Parameters.FOCUS_MODE_CONTINUOUS_PICTURE);
        camera.setParameters(parameters);
        camera.cancelAutoFocus();
    }

    private void disableContinuousAutoFocus(Camera camera) {
        Camera.Parameters parameters = camera.getParameters();
        parameters.setFocusMode(Camera.Parameters.FOCUS_MODE_AUTO);
        camera.setParameters(parameters);
        camera.cancelAutoFocus();
    }

    public void turnOffAutoFocus(Camera camera){
        camera.cancelAutoFocus();
        if(currentAutoFocus == CameraManager.AutoFocusMode.TIMER_MODE){
            stopTimerFocusing();
        }
    }

    public View.OnClickListener getSingleAutoFocusOnClick(final Camera camera) {
        return new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                requestSingleAutoFocus(camera);
            }
        };
    }

    public static class AutoFocusException extends CameraException{

        public AutoFocusException() {
            super("Can not start autofocus!");
        }
    }
}
