package pl.primesoft.aztecreaderdemo.Scanning.CameraManager.Utils;

import android.hardware.Camera;

import java.util.List;

/**
 * Created by wygralak on 2016-03-16.
 */
public class FpsRangeUtils {
    private static int[] defaultRange = null;

    public static boolean isFpsRangeAvailable(Camera camera) {
        List<int[]> fpsValues = camera.getParameters().getSupportedPreviewFpsRange();
        return (fpsValues != null && fpsValues.size() > 0);
    }

    public static void setMaximumFpsRange(Camera camera) {
        updateDefaultRange(camera);
        int[] maxRange = getMaxFpsRange(camera);
        if (maxRange != null) {
            Camera.Parameters parameters = camera.getParameters();
            parameters.setPreviewFpsRange(maxRange[0], maxRange[1]);
            camera.setParameters(parameters);
        }
    }

    public static void setMinimumFpsRange(Camera camera) {
        updateDefaultRange(camera);
        int[] minRange = getMinFpsRange(camera);
        if (minRange != null) {
            Camera.Parameters parameters = camera.getParameters();
            parameters.setPreviewFpsRange(minRange[0], minRange[1]);
            camera.setParameters(parameters);
        }
    }

    public static void setDefaultFpsRange(Camera camera) {
        if (defaultRange == null) {
            updateDefaultRange(camera);
        } else {
            Camera.Parameters parameters = camera.getParameters();
            parameters.setPreviewFpsRange(defaultRange[0], defaultRange[1]);
            camera.setParameters(parameters);
        }
    }

    private static int[] getMaxFpsRange(Camera camera) {
        List<int[]> ranges = camera.getParameters().getSupportedPreviewFpsRange();
        if (ranges == null || ranges.size() < 1) {
            return null;
        }
        int[] currMax = ranges.get(0);
        int currMaxDiff = currMax[1] - currMax[0];
        for (int i = 1; i < ranges.size(); i++) {
            int[] rangeIth = ranges.get(i);
            if (rangeIth[1] > currMax[1]) {
                currMax = rangeIth;
                currMaxDiff = rangeIth[1] - rangeIth[0];
            } else if (rangeIth[1] == currMax[1]) {
                int diff = rangeIth[1] - rangeIth[0];
                if (diff < currMaxDiff) {
                    currMax = rangeIth;
                    currMaxDiff = diff;
                }
            }
        }
        return currMax;
    }

    private static int[] getMinFpsRange(Camera camera) {
        List<int[]> ranges = camera.getParameters().getSupportedPreviewFpsRange();
        if (ranges == null || ranges.size() < 1) {
            return null;
        }
        int[] currMin = ranges.get(0);
        int currMinDiff = currMin[1] - currMin[0];
        for (int i = 1; i < ranges.size(); i++) {
            int[] rangeIth = ranges.get(i);
            if (rangeIth[1] < currMin[1]) {
                currMin = rangeIth;
                currMinDiff = rangeIth[1] - rangeIth[0];
            } else if (rangeIth[1] == currMin[1]) {
                int diff = rangeIth[1] - rangeIth[0];
                if (diff < currMinDiff) {
                    currMin = rangeIth;
                    currMinDiff = diff;
                }
            }
        }
        return currMin;
    }

    private static void updateDefaultRange(Camera camera) {
        if (defaultRange == null) {
            defaultRange = new int[2];
            camera.getParameters().getPreviewFpsRange(defaultRange);
        }
    }
}
