package pl.primesoft.aztecreaderdemo.Scanning;

import android.graphics.ImageFormat;
import android.hardware.Camera;

import java.util.concurrent.Semaphore;

import pl.primesoft.pspbarcodereader.PspBarcodeReader;

/**
 * Created by wygralak on 2016-03-16.
 */
public class RecognizingThread extends Thread implements Camera.PreviewCallback {
    private final PspBarcodeReader br;
    private byte[] data;
    private final int width;
    private final int height;
    private final IRecognisingCallback callback;
    private Semaphore semaphore;
    private boolean shouldStop = false;

    public RecognizingThread(PspBarcodeReader br, int width, int height,
                             IRecognisingCallback callback) {
        this.br = br;
        this.width = width;
        this.height = height;
        this.callback = callback;
        semaphore = new Semaphore(0);
    }

    @Override
    public void run() {
        while (true) {
            if (shouldStop) {
                break;
            }
            callback.requestPreview(this);
            try {
                semaphore.acquire();
            } catch (InterruptedException e) {
                e.printStackTrace();
                continue;
            }
            if (shouldStop) {
                break;
            }
            String result = br.recognize(data,
                    width,
                    height);
            if (result != null && result.length() > 0) {
                callback.onRecognizeSuccess(result);
            } else {
                callback.onRecognizeFailure();
            }
        }
    }

    public void performPreviewCallbackWithBuffer(Camera camera) {
        if (data == null) {
            float bytePerPixel = (float) ImageFormat.getBitsPerPixel(camera.getParameters().getPreviewFormat()) / 8.0f;
            int previewByteSize = (int) Math.ceil((width * height) * bytePerPixel * 1.2f); //+20% na wszelki wypadek
            data = new byte[previewByteSize];
        }
        camera.addCallbackBuffer(data);
        camera.setPreviewCallbackWithBuffer(this);
    }

    @Override
    public void onPreviewFrame(byte[] data, Camera camera) {
        camera.setPreviewCallback(null);
        callback.notifyPreviewCallbackReleased();
        semaphore.release();
    }

    public void shouldStop() {
        shouldStop = true;
        semaphore.release();
    }

    public interface IRecognisingCallback {

        void requestPreview(RecognizingThread thread);

        void onRecognizeSuccess(String result);

        void onRecognizeFailure();

        void notifyPreviewCallbackReleased();
    }

}
