package pl.primesoft.aztecreaderdemo.Scanning;

import android.Manifest;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.res.Resources;
import android.hardware.Camera;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import com.google.android.material.snackbar.Snackbar; //import android.support.v4.design.widget.Snackbar;
import androidx.core.app.ActivityCompat; //import android.support.v4.app.ActivityCompat;
import androidx.core.content.ContextCompat; //import android.support.v4.content.ContextCompat;
import android.view.KeyCharacterMap;
import android.view.KeyEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.RelativeLayout;

import java.util.ArrayList;
import java.util.List;

import pl.primesoft.aztecreaderdemo.R;
import pl.primesoft.aztecreaderdemo.Scanning.CameraManager.CameraManager;
import pl.primesoft.aztecreaderdemo.Scanning.CameraManager.Utils.AutoFocusUtils;
import pl.primesoft.aztecreaderdemo.Views.AboutDialog;
import pl.primesoft.aztecreaderdemo.Views.SelfMeasuringCameraView;
import pl.primesoft.pspbarcodereader.PspBarcodeReader;

public class ScanningActivity extends Activity {
    /*
        PARAMETERS TO SET
     */

    private static final CameraManager.AutoFocusMode CURRENT_AUTO_FOCUS_MODE = CameraManager.AutoFocusMode.CONTINUOUS_MODE;
    private static final CameraManager.FpsRangeMode CURRENT_FPS_RANGE_MODE = CameraManager.FpsRangeMode.DEFAULT;
    private static final boolean USE_ZOOMING_IF_SUPPORTED = false;
    private static final boolean USE_FLASH_IF_SUPPORTED = true;
    private static final boolean USE_STABILIZATION_IF_SUPPORTED = true;

    private static final int MAX_THREADS_NUMBER = 2;

    //-----------

    private static final int CAMERA_PERMISSION_REQUEST = 13;

    protected RelativeLayout previewFront;
    private ImageView flashBtn;
    private ImageView zoomBtn;
    protected ViewGroup cameraPreviewContainer;
    private SelfMeasuringCameraView cameraPreview;

    private CameraManager cameraManager;

    private boolean recognized = false;
    private boolean callbackFree = true;
    private List<RecognizingThread> recognizingThreads = new ArrayList<>(MAX_THREADS_NUMBER);
    private List<RecognizingThread> previewQueue = new ArrayList<>();

    static {
        System.loadLibrary("azteccodereaderandroidwrapper");
    }

    private PspBarcodeReader br = PspBarcodeReader.create();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_scanning_with_camera_manager);
        cameraManager = new CameraManager(this);
        initViews();

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            checkForCameraPermission();
        }
    }

    private void initViews() {
        cameraPreviewContainer = (ViewGroup) findViewById(R.id.cameraPreviewContainer);
        previewFront = (RelativeLayout) findViewById(R.id.previewFront);
        flashBtn = (ImageView) findViewById(R.id.flashBtn);
        zoomBtn = (ImageView) findViewById(R.id.zoomBtn);
        initStatusBarBackground();

        if (!haveHardwareKeys()) {
            FrameLayout infoWrapper = (FrameLayout) findViewById(R.id.infoWrapper);
            infoWrapper.setPadding(infoWrapper.getPaddingLeft(), infoWrapper.getPaddingTop(),
                    infoWrapper.getPaddingRight(), infoWrapper.getPaddingBottom() + getNavBarHeight());
        }

        View infoButton = findViewById(R.id.infoBtn);
        infoButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                recognized = true;
                AboutDialog dialog = new AboutDialog(ScanningActivity.this) {
                    @Override
                    public void navigateToWebsite(String url) {
                        navigateToWebsiteIntent(url);
                    }
                };
                dialog.setOnDismissListener(new DialogInterface.OnDismissListener() {
                    @Override
                    public void onDismiss(DialogInterface dialog) {
                        recognized = false;
                        invokePreviewCallbackFromQueue();
                    }
                });
                dialog.show();
            }
        });
    }

    private void navigateToWebsiteIntent(String url) {
        Intent intent = new Intent(Intent.ACTION_VIEW, Uri.parse(url));
        startActivity(intent);
    }

    private void initStatusBarBackground() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
            View statusBarBackground = findViewById(R.id.statusBarBackground);
            ViewGroup.LayoutParams layoutParams = statusBarBackground.getLayoutParams();
            layoutParams.height = getStatusBarHeight();
            statusBarBackground.setLayoutParams(layoutParams);
        }
    }

    private int getStatusBarHeight() {
        int result = 0;
        int resourceId = getResources().getIdentifier("status_bar_height", "dimen", "android");
        if (resourceId > 0) {
            result = getResources().getDimensionPixelSize(resourceId);
        }
        return result;
    }

    public int getNavBarHeight() {
        Resources res = getResources();
        int resId = res.getIdentifier("navigation_bar_height", "dimen", "android");
        if (resId > 0) {
            return res.getDimensionPixelSize(resId);
        }
        return 0;
    }

    private boolean haveHardwareKeys() {
        return KeyCharacterMap.deviceHasKey(KeyEvent.KEYCODE_BACK) && KeyCharacterMap.deviceHasKey(KeyEvent.KEYCODE_HOME);
    }

    @Override
    protected void onResume() {
        super.onResume();
        try {
            cameraManager.setupCamera();
            View.OnClickListener focusOnClick = null;
            try {
                focusOnClick = cameraManager.setupAutoFocus(CURRENT_AUTO_FOCUS_MODE);
            } catch (AutoFocusUtils.AutoFocusException e) {
                e.printStackTrace();
            }
            setupCameraView();
            if (focusOnClick != null) {
                cameraPreview.setOnClickListener(focusOnClick);
            }
            setupZooming();
            setupFlashlight();

            if (cameraManager.isFpsRangeAvailable()) {
                cameraManager.setFpsRangeMode(CURRENT_FPS_RANGE_MODE);
            }
            if (USE_STABILIZATION_IF_SUPPORTED && cameraManager.isVideoStabilizationSupported()) {
                cameraManager.setVideoStabilization(true);
            }
        } catch (Exception e) {
            e.printStackTrace();
            showError(e);
        }
    }

    private void setupZooming() {
        if (USE_ZOOMING_IF_SUPPORTED && cameraManager.isZoomAvailable()) {
            View.OnClickListener zoomingOnClick = cameraManager.setupZooming();
            if (zoomingOnClick != null) {
                zoomBtn.setOnClickListener(zoomingOnClick);
                zoomBtn.setVisibility(View.VISIBLE);
            } else {
                zoomBtn.setVisibility(View.GONE);
            }
        }
    }

    private void setupFlashlight() {
        if (USE_FLASH_IF_SUPPORTED && cameraManager.isFlashAvailable()) {
            flashBtn.setVisibility(View.VISIBLE);
            flashBtn.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    String newFlash = cameraManager.toggleFlash();
                    if (newFlash == null) {
                        showError("Error while using flashlight!");
                    } else if (newFlash.equals(Camera.Parameters.FLASH_MODE_OFF)) {
                        flashBtn.setImageResource(R.drawable.flash_off_black_36dp);
                    } else {
                        flashBtn.setImageResource(R.drawable.flash_on_black_36dp);
                    }
                }
            });
        } else {
            flashBtn.setVisibility(View.GONE);
        }
    }

    private void setupCameraView() {
        cameraPreview = new SelfMeasuringCameraView(this, cameraManager.getCameraInstance());
        cameraPreview.post(new Runnable() {
            @Override
            public void run() {
                while (cameraPreview.getHeight() < 0) {
                    try {
                        Thread.sleep(50);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                }
                setupPreviewCallback();
                ViewGroup.LayoutParams layoutParams = previewFront.getLayoutParams();
                layoutParams.height = cameraPreview.getHeight();
                layoutParams.width = cameraPreview.getWidth();
                previewFront.setLayoutParams(layoutParams);
            }
        });
        cameraPreviewContainer.addView(cameraPreview);
    }

    private void setupPreviewCallback() {
        RecognizingThread.IRecognisingCallback callback = new RecognizingThread.IRecognisingCallback() {
            @Override
            public void requestPreview(RecognizingThread previewCallback) {
                if (callbackFree && !recognized) {
                    callbackFree = false;
                    previewCallback.performPreviewCallbackWithBuffer(cameraManager.getCameraInstance());
                } else {
                    previewQueue.add(previewCallback);
                }
            }

            @Override
            public void onRecognizeSuccess(String result) {
                if (!recognized) {
                    recognized = true;
                    decodingSuccess(result);
                }
            }

            @Override
            public void onRecognizeFailure() {

            }

            @Override
            public void notifyPreviewCallbackReleased() {
                invokePreviewCallbackFromQueue();
            }
        };
        try {
            Camera.Size size = cameraManager.getCameraInstance().getParameters().getPreviewSize();
            for (int i = 0; i < MAX_THREADS_NUMBER; i++) {
                RecognizingThread t = new RecognizingThread(br, size.width, size.height, callback);
                recognizingThreads.add(t);
                t.start();
            }
        } catch (Exception e) {
            //Possible Exception: Camera is being used after Camera.release()
            e.printStackTrace();
        }
    }

    private void invokePreviewCallbackFromQueue() {
        if (previewQueue.size() > 0 && !recognized) {
            callbackFree = false;
            RecognizingThread recoThread = previewQueue.get(0);
            previewQueue.remove(0);
            recoThread.performPreviewCallbackWithBuffer(cameraManager.getCameraInstance());
        } else {
            callbackFree = true;
        }
    }


    private void decodingSuccess(final String result) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                showRecognizedDialog(result);
            }
        });
    }

    private void showRecognizedDialog(String result) {
        AlertDialog dialog = new AlertDialog.Builder(this)
                .setTitle("Recognized!")
                .setMessage(result)
                .setCancelable(false)
                .setOnKeyListener(new DialogInterface.OnKeyListener() {
                    @Override
                    public boolean onKey(DialogInterface dialog, int keyCode, KeyEvent event) {
                        if (keyCode == KeyEvent.KEYCODE_BACK) {
                            if (event.getAction() == KeyEvent.ACTION_UP) {
                                recognized = false;
                                dialog.dismiss();
                                if (callbackFree) {
                                    invokePreviewCallbackFromQueue();
                                }
                                return true;
                            }
                        }
                        return false;
                    }
                })
                .setNegativeButton("Close", new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int which) {
                        recognized = false;
                        if (callbackFree) {
                            invokePreviewCallbackFromQueue();
                        }
                    }
                })
                .create();
        dialog.show();
    }

    private void checkForCameraPermission() {
        if (ContextCompat.checkSelfPermission(this,
                Manifest.permission.CAMERA)
                != PackageManager.PERMISSION_GRANTED) {

            ActivityCompat.requestPermissions(this,
                    new String[]{Manifest.permission.CAMERA},
                    CAMERA_PERMISSION_REQUEST);
        }
    }

    public void showError(Exception e) {
        Throwable cause = e.getCause();
        if (cause != null) {
            showError(String.valueOf(cause.getMessage()));
        } else {
            showError(String.valueOf(e.getMessage()));
        }
    }

    public void showError(String msg) {
        //Toast.makeText(this, msg, Toast.LENGTH_LONG).show();
        Snackbar.make(cameraPreviewContainer, msg, Snackbar.LENGTH_LONG).show();
    }

    @Override
    protected void onPause() {
        Camera camera = cameraManager.getCameraInstance();
        if (camera != null) {
            cameraManager.getCameraInstance().stopPreview();
        }
        closeRecognizingThreads();
        cameraManager.turnOffCamera();
        cameraPreviewContainer.removeAllViews();
        super.onPause();
    }

    private void closeRecognizingThreads() {
        for (int i = 0; i < recognizingThreads.size(); i++) {
            recognizingThreads.get(i).shouldStop();
        }
        recognizingThreads.clear();
        previewQueue.clear();
        callbackFree = true;
    }
}
