package pl.primesoft.aztecreaderdemo.Views;

import android.content.Context;
import android.hardware.Camera;
import android.util.Log;
import android.util.Pair;
import android.view.SurfaceHolder;
import android.view.SurfaceView;

import java.util.List;

/**
 * Created by wygralak on 2016-01-25.
 */
public class SelfMeasuringCameraView extends SurfaceView implements SurfaceHolder.Callback {

    private SurfaceHolder mHolder;
    private Camera mCamera;
    private List<Camera.Size> mSupportedPreviewSizes;
    private Camera.Size mPreviewSize;

    public SelfMeasuringCameraView(Context context, Camera camera) {
        super(context);
        mCamera = camera;

        mSupportedPreviewSizes = mCamera.getParameters().getSupportedPreviewSizes();
        filterResolutions(mSupportedPreviewSizes);

        mHolder = getHolder();
        mHolder.addCallback(this);
        // deprecated setting, but required on Android versions prior to 3.0
        mHolder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);

    }

    protected void filterResolutions(List<Camera.Size> mSupportedPreviewSizes) {
        for (int i = 0; i < mSupportedPreviewSizes.size(); i++) {
            Camera.Size sizeIth = mSupportedPreviewSizes.get(i);
            if (sizeIth.width < 400 || sizeIth.height < 400) {
                mSupportedPreviewSizes.remove(i);
                i--;
                continue;
            }
            if (sizeIth.width > 1280 || sizeIth.height > 1280) {
                mSupportedPreviewSizes.remove(i);
                i--;
            }
        }
    }

    public void surfaceCreated(SurfaceHolder holder) {
    }

    public void surfaceDestroyed(SurfaceHolder holder) {
    }

    public void surfaceChanged(SurfaceHolder holder, int format, int w, int h) {
        if (mHolder.getSurface() == null) {
            return;
        }

        try {
            mCamera.stopPreview();
        } catch (Exception e) {
            // ignore: tried to stop a non-existent preview
        }

        // set preview size and make any resize, rotate or reformatting changes here
        // start preview with new settings
        try {
            Camera.Parameters parameters = mCamera.getParameters();
            parameters.setPreviewSize(mPreviewSize.width, mPreviewSize.height);
            mCamera.setParameters(parameters);
            mCamera.setPreviewDisplay(mHolder);
            mCamera.startPreview();

        } catch (Exception e) {
            Log.d("Camera Preview", "Error starting camera preview: " + e.getMessage());
        }
    }

    @Override
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
        int width = resolveSize(getSuggestedMinimumWidth(), widthMeasureSpec);
        int height = resolveSize(getSuggestedMinimumHeight(), heightMeasureSpec);
        boolean swaped = false;

        if (width < height) {
            int tmp = width;
            width = height;
            height = tmp;
            swaped = true;
        }

        mPreviewSize = getOptimalPreviewSize(mSupportedPreviewSizes, width, height);
        if (mPreviewSize != null) {
            if (swaped) {
                setMeasuredDimension(height, width);
            } else {
                setMeasuredDimension(width, height);
            }
        } else {
            mPreviewSize = getBestPreviewSize(mSupportedPreviewSizes, width, height);
            Pair<Integer, Integer> viewSize = getValidWindowSize(mPreviewSize, width, height);
            if (swaped) {
                setMeasuredDimension(viewSize.second, viewSize.first);
            } else {
                setMeasuredDimension(viewSize.first, viewSize.second);
            }
        }
    }

    private Pair<Integer, Integer> getValidWindowSize(Camera.Size previewSize, int width, int height) {
        double widthRatio = (double) width / previewSize.width;
        double heightRatio = (double) height / previewSize.height;
        double superRatio = widthRatio > heightRatio ? widthRatio : heightRatio;
        int outHeight = (int) (((double) previewSize.height) * superRatio);
        int outWidth = (int) (((double) previewSize.width) * superRatio);
        return new Pair<>(outWidth, outHeight);

    }

    private Camera.Size getOptimalPreviewSize(List<Camera.Size> sizes, int targetWidth, int targetHeight) {
        final double ASPECT_TOLERANCE = 0.05;
        double targetRatio = (double) targetHeight / targetWidth;

        if (sizes == null)
            return null;

        Camera.Size optimalSize = null;

        for (Camera.Size size : sizes) {
            double ratio = (double) size.height / size.width;
            if (Math.abs(ratio - targetRatio) > ASPECT_TOLERANCE) {
                continue;
            }

            if (optimalSize == null || optimalSize.height > size.height) {
                optimalSize = size;
            }
        }
        return optimalSize;
    }

    private Camera.Size getBestPreviewSize(List<Camera.Size> sizes, int targetWidth, int targetHeight) {
        double ASPECT_TOLERANCE = 0.05;
        double targetRatio = (double) targetHeight / targetWidth;

        Camera.Size optimalSize = null;
        double minDiff = Double.MAX_VALUE;
        for (Camera.Size size : sizes) {
            double ratio = (double) size.height / size.width;
            if (Math.abs(ratio - targetRatio) < minDiff + ASPECT_TOLERANCE) {
                if (optimalSize == null || optimalSize.height > size.height) {
                    optimalSize = size;
                    minDiff = Math.abs(ratio - targetRatio);
                }
            }
        }
        return optimalSize;
    }
}
